#![allow(non_snake_case)]
use dioxus::prelude::*;

use bottomify::bottom::{encode_string, decode_string};

fn main() {
    dioxus_web::launch(App);
}

fn App(cx: Scope) -> Element {
    cx.render(rsx! {
        Layout {
            Encoder { }
        }
    })
}

#[derive(Props)]
struct LayoutProps<'a> {
    children: Element<'a>,
}
fn Layout<'a>(cx: Scope<'a, LayoutProps<'a>>) -> Element {
    cx.render(rsx! {
        nav {
            h1 {
                "bottom-rs-web"
            }
            a {
                href: "https://github.com/bottom-software-foundation/bottom-rs",
                target: "_blank",
                "original tool"
            }
        }
        main {
            &cx.props.children
        }
        footer {
            span {
                "Created by "
                a {
                    href: "https://tempest.dev/about/rose",
                    "Ashelyn Rose"
                },
                " because they were far too bored"
            }
        }
    })
}

fn Encoder<'a>(cx: Scope<'a, LayoutProps<'a>>) -> Element {
    let input_text = use_state(cx, || "".to_string());
    let status_text = use_state(cx, || "".to_string());
    let error_status = use_state(cx, || false);

    let input_stripped = strip_spaces(input_text.to_string());
    let is_valid_bottom = decode_string(&input_stripped).is_ok();

    let bottomify = move |event: Event<MouseData>| {
        event.stop_propagation();

        let input = input_text.to_string();
        let space_stripped = strip_spaces(input_text.to_string());

        if space_stripped.len() < 1 {
            return
        }

        input_text.set(encode_string(&input_text.to_string()));
        status_text.set(format!("Bottomed from \"{}\"", input));
        error_status.set(false);
    };

    let debottomify = move |event: Event<MouseData>| {
        event.stop_propagation();

        let input = input_text.to_string();
        let space_stripped = strip_spaces(input_text.to_string());

        if space_stripped.len() < 1 {
            return
        }

        let result = decode_string(&space_stripped);

        match result {
            Ok(output) => {
                input_text.set(output);
                status_text.set(format!("De-bottomed from {}", input));
                error_status.set(false);
            }
            Err(_error) => {
                status_text.set("Could not decode bottom text".to_string());
                error_status.set(true);
            }
        }
    };

    cx.render(rsx! {
        label {
            "for": "input",
            "Enter a message to convert:"
        }
        form {
            action: "javascript:void(0);",
            textarea {
                id: "input",
                value: "{input_text}",
                oninput: move |evt| input_text.set(evt.value.clone()),
            }
            button {
                disabled: is_valid_bottom,
                title: if is_valid_bottom {
                    "Not encoding, input is already valid bottom text"
                } else {
                    ""
                },
                onclick: bottomify,
                "Bottomify"
            }
            button {
                disabled: !is_valid_bottom,
                title: if !is_valid_bottom {
                    "Cannot decode, input is not valid bottom text"
                } else {
                    ""
                },
                onclick: debottomify,
                "Debottomify"
            }
        }

        span {
            id: "status",
            class: if **error_status {
                "error"
            } else {
                "false"
            },
            "{status_text}"
        }
    })
}

fn strip_spaces(input: String) -> String {
    input.split(' ')
        .collect::<Vec<_>>()
        .join("")
}
